"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Consumer = void 0;
const redis_smq_common_1 = require("redis-smq-common");
const index_js_1 = require("../../../common/index.js");
const redis_client_js_1 = require("../../../common/redis-client/redis-client.js");
const index_js_2 = require("../../../config/index.js");
const _parse_queue_extended_params_js_1 = require("../../queue/_/_parse-queue-extended-params.js");
const consumer_heartbeat_js_1 = require("../consumer-heartbeat/consumer-heartbeat.js");
const message_handler_runner_js_1 = require("../message-handler-runner/message-handler-runner.js");
const multiplexed_message_handler_runner_js_1 = require("../message-handler-runner/multiplexed-message-handler-runner.js");
const event_bus_publisher_js_1 = require("./event-bus-publisher.js");
class Consumer extends redis_smq_common_1.Runnable {
    constructor(enableMultiplexing) {
        super();
        this.eventBus = null;
        this.heartbeat = null;
        this.onRedisError = (error) => {
            this.logger.error(`Redis client error: ${error.message}`);
            this.handleError(error);
        };
        this.setUpHeartbeat = (cb) => {
            var _a;
            this.logger.debug('Setting up consumer heartbeat');
            this.heartbeat = new consumer_heartbeat_js_1.ConsumerHeartbeat(this, this.redisClient, (_a = this.eventBus) !== null && _a !== void 0 ? _a : null);
            this.heartbeat.on('consumerHeartbeat.error', (err) => {
                this.logger.error(`Heartbeat error: ${err.message}`);
                this.handleError(err);
            });
            this.logger.debug('Starting heartbeat');
            this.heartbeat.run((err) => {
                if (err) {
                    this.logger.error(`Failed to start heartbeat: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.debug('Heartbeat started successfully');
                    cb();
                }
            });
        };
        this.shutDownHeartbeat = (cb) => {
            if (this.heartbeat) {
                this.logger.debug('Shutting down heartbeat');
                this.heartbeat.shutdown((err) => {
                    if (err) {
                        this.logger.warn(`Error during heartbeat shutdown: ${err.message}`);
                    }
                    this.logger.debug('Heartbeat shut down');
                    this.heartbeat = null;
                    cb();
                });
            }
            else {
                this.logger.debug('No heartbeat to shut down');
                cb();
            }
        };
        this.runMessageHandlers = (cb) => {
            this.logger.debug('Starting message handlers');
            this.messageHandlerRunner.run((err) => {
                if (err) {
                    this.logger.error(`Failed to start message handlers: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.debug('Message handlers started successfully');
                    cb();
                }
            });
        };
        this.shutdownMessageHandlers = (cb) => {
            this.logger.debug('Shutting down message handlers');
            this.messageHandlerRunner.shutdown((err) => {
                if (err) {
                    this.logger.warn(`Error during message handlers shutdown: ${err.message}`);
                }
                this.logger.debug('Message handlers shut down');
                cb();
            });
        };
        this.shutDownRedisClient = (cb) => {
            this.logger.debug('Shutting down Redis client');
            this.redisClient.shutdown((err) => {
                if (err) {
                    this.logger.warn(`Error during Redis client shutdown: ${err.message}`);
                }
                this.logger.debug('Redis client shut down');
                cb();
            });
        };
        const config = index_js_2.Configuration.getSetConfig();
        this.logger = redis_smq_common_1.logger.getLogger(config.logger, this.constructor.name.toLowerCase());
        this.logger.info(`Initializing consumer${enableMultiplexing ? ' with multiplexing enabled' : ''}`);
        this.redisClient = new redis_client_js_1.RedisClient();
        this.redisClient.on('error', this.onRedisError);
        this.logger.debug('Redis client initialized');
        if (index_js_2.Configuration.getSetConfig().eventBus.enabled) {
            this.logger.debug('Event bus is enabled, initializing');
            this.eventBus = new index_js_1.EventBus();
            this.eventBus.on('error', this.onRedisError);
            (0, event_bus_publisher_js_1.eventBusPublisher)(this, this.eventBus, this.logger);
            this.logger.debug('Event bus publisher configured');
        }
        else {
            this.logger.debug('Event bus is disabled');
        }
        this.logger.debug(`Creating ${enableMultiplexing ? 'multiplexed' : 'standard'} message handler runner`);
        this.messageHandlerRunner = enableMultiplexing
            ? new multiplexed_message_handler_runner_js_1.MultiplexedMessageHandlerRunner(this, this.redisClient, this.eventBus)
            : new message_handler_runner_js_1.MessageHandlerRunner(this, this.redisClient, this.eventBus);
        this.messageHandlerRunner.on('consumer.messageHandlerRunner.error', (err) => {
            this.logger.error(`Message handler runner error: ${err.message}`);
            this.handleError(err);
        });
        this.logger.info(`Consumer initialized with ID: ${this.id}`);
    }
    goingUp() {
        this.logger.info('Consumer going up');
        return super.goingUp().concat([
            (cb) => {
                if (this.eventBus) {
                    this.logger.debug('Initializing event bus');
                    this.eventBus.init((err) => {
                        if (err) {
                            this.logger.error(`Failed to initialize event bus: ${err.message}`);
                        }
                        else {
                            this.logger.debug('Event bus initialized');
                        }
                        cb(err);
                    });
                }
                else {
                    this.logger.debug('Skipping event bus initialization (disabled)');
                    cb();
                }
            },
            (cb) => {
                this.logger.debug(`Emitting consumer.goingUp event for consumer ${this.id}`);
                this.emit('consumer.goingUp', this.id);
                cb();
            },
            this.redisClient.init,
            this.setUpHeartbeat,
            this.runMessageHandlers,
        ]);
    }
    goingDown() {
        this.logger.info('Consumer going down');
        this.logger.debug(`Emitting consumer.goingDown event for consumer ${this.id}`);
        this.emit('consumer.goingDown', this.id);
        return [
            this.shutdownMessageHandlers,
            this.shutDownHeartbeat,
            this.shutDownRedisClient,
        ].concat(super.goingDown());
    }
    up(cb) {
        this.logger.info('Consumer is up');
        super.up(() => {
            this.logger.debug(`Emitting consumer.up event for consumer ${this.id}`);
            this.emit('consumer.up', this.id);
            cb(null, true);
        });
    }
    down(cb) {
        this.logger.info('Consumer is down');
        super.down(() => {
            this.logger.debug(`Emitting consumer.down event for consumer ${this.id}`);
            this.emit('consumer.down', this.id);
            if (this.eventBus) {
                this.logger.debug('Delaying event bus shutdown by 1000ms');
                setTimeout(() => {
                    var _a;
                    this.logger.debug('Shutting down event bus');
                    (_a = this.eventBus) === null || _a === void 0 ? void 0 : _a.shutdown((err) => {
                        if (err) {
                            this.logger.warn(`Error during event bus shutdown: ${err.message}`);
                        }
                        this.logger.debug('Event bus shut down');
                        cb(null, true);
                    });
                }, 1000);
            }
            else {
                this.logger.debug('No event bus to shut down');
                cb(null, true);
            }
        });
    }
    getLogger() {
        return this.logger;
    }
    handleError(err) {
        this.logger.error(`Consumer error: ${err.message}`, err);
        this.logger.debug(`Emitting consumer.error event for consumer ${this.id}`);
        this.emit('consumer.error', err, this.id);
        super.handleError(err);
    }
    consume(queue, messageHandler, cb) {
        this.logger.info(`Setting up consumption for queue: ${typeof queue === 'string' ? queue : JSON.stringify(queue)}`);
        const parsedQueueParams = (0, _parse_queue_extended_params_js_1._parseQueueExtendedParams)(queue);
        if (parsedQueueParams instanceof Error) {
            this.logger.error(`Failed to parse queue parameters: ${parsedQueueParams.message}`);
            cb(parsedQueueParams);
        }
        else {
            this.logger.debug(`Adding message handler for queue: ${JSON.stringify(parsedQueueParams)}`);
            this.messageHandlerRunner.addMessageHandler(parsedQueueParams, messageHandler, (err) => {
                if (err) {
                    this.logger.error(`Failed to add message handler: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.info(`Successfully set up consumption for queue: ${parsedQueueParams.queueParams.name} (namespace: ${parsedQueueParams.queueParams.ns}${parsedQueueParams.groupId ? `, group: ${parsedQueueParams.groupId}` : ''})`);
                    cb();
                }
            });
        }
    }
    cancel(queue, cb) {
        this.logger.info(`Canceling consumption for queue: ${typeof queue === 'string' ? queue : JSON.stringify(queue)}`);
        const parsedQueueParams = (0, _parse_queue_extended_params_js_1._parseQueueExtendedParams)(queue);
        if (parsedQueueParams instanceof Error) {
            this.logger.error(`Failed to parse queue parameters: ${parsedQueueParams.message}`);
            cb(parsedQueueParams);
        }
        else {
            this.logger.debug(`Removing message handler for queue: ${JSON.stringify(parsedQueueParams)}`);
            this.messageHandlerRunner.removeMessageHandler(parsedQueueParams, (err) => {
                if (err) {
                    this.logger.error(`Failed to remove message handler: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.info(`Successfully canceled consumption for queue: ${parsedQueueParams.queueParams.name} (namespace: ${parsedQueueParams.queueParams.ns}${parsedQueueParams.groupId ? `, group: ${parsedQueueParams.groupId}` : ''})`);
                    cb();
                }
            });
        }
    }
    getQueues() {
        this.logger.debug('Getting list of queues being consumed');
        const queues = this.messageHandlerRunner.getQueues();
        this.logger.debug(`Consumer is handling ${queues.length} queues: ${JSON.stringify(queues)}`);
        return queues;
    }
}
exports.Consumer = Consumer;
//# sourceMappingURL=consumer.js.map