"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConsumeMessage = void 0;
const fs_1 = require("fs");
const path_1 = __importDefault(require("path"));
const redis_smq_common_1 = require("redis-smq-common");
const scripts_js_1 = require("../../../../../common/redis-client/scripts/scripts.js");
const redis_keys_js_1 = require("../../../../../common/redis-keys/redis-keys.js");
const index_js_1 = require("../../../../../config/index.js");
const index_js_2 = require("../../../../message/index.js");
const index_js_3 = require("../../../types/index.js");
const index_js_4 = require("../../errors/index.js");
const message_unacknowledgement_js_1 = require("./message-unacknowledgement.js");
const event_bus_publisher_js_1 = require("./event-bus-publisher.js");
class ConsumeMessage extends redis_smq_common_1.Runnable {
    constructor(redisClient, consumer, queue, messageHandlerId, messageHandler, eventBus) {
        super();
        this.consumeMessageWorker = null;
        this.validateMessageHandler = (cb) => {
            if (typeof this.messageHandler === 'string') {
                this.logger.debug(`Validating message handler file: ${this.messageHandler}`);
                const fileExtension = path_1.default.extname(this.messageHandler);
                if (!['.js', '.cjs'].includes(fileExtension)) {
                    this.logger.error(`Invalid message handler file extension: ${fileExtension}, expected .js or .cjs`);
                    cb(new index_js_4.ConsumerMessageHandlerFilenameExtensionError());
                }
                else {
                    this.logger.debug(`Checking if message handler file exists: ${this.messageHandler}`);
                    (0, fs_1.stat)(this.messageHandler, (err) => {
                        if (err) {
                            this.logger.error(`Message handler file not found: ${this.messageHandler}, error: ${err.message}`);
                            cb(new index_js_4.ConsumerMessageHandlerFileError());
                        }
                        else {
                            this.logger.debug(`Message handler file validated successfully: ${this.messageHandler}`);
                            cb();
                        }
                    });
                }
            }
            else {
                this.logger.debug('Using function-based message handler, no file validation needed');
                cb();
            }
        };
        this.queue = queue;
        this.consumerId = consumer.getId();
        this.messageHandler = messageHandler;
        this.messageHandlerId = messageHandlerId;
        this.redisClient = redisClient;
        this.messageUnack = new message_unacknowledgement_js_1.MessageUnacknowledgement(redisClient);
        this.logger = redis_smq_common_1.logger.getLogger(index_js_1.Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
        this.logger.debug(`Initializing ConsumeMessage for consumer ${this.consumerId}, queue ${JSON.stringify(this.queue)}, messageHandlerId ${this.messageHandlerId}`);
        const { keyQueueProcessing } = redis_keys_js_1.redisKeys.getQueueConsumerKeys(this.queue.queueParams, this.consumerId);
        const { keyQueueAcknowledged } = redis_keys_js_1.redisKeys.getQueueKeys(this.queue.queueParams, this.queue.groupId);
        this.keyQueueAcknowledged = keyQueueAcknowledged;
        this.keyQueueProcessing = keyQueueProcessing;
        this.logger.debug(`Queue processing key: ${this.keyQueueProcessing}, acknowledged key: ${this.keyQueueAcknowledged}`);
        if (eventBus) {
            this.logger.debug('Event bus provided, setting up event bus publisher');
            (0, event_bus_publisher_js_1.eventBusPublisher)(this, eventBus, this.logger);
        }
        else {
            this.logger.debug('No event bus provided, skipping event bus publisher setup');
        }
        this.logger.info(`ConsumeMessage initialized for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
    }
    getLogger() {
        return this.logger;
    }
    acknowledgeMessage(message) {
        const messageId = message.getId();
        this.logger.debug(`Acknowledging message ${messageId}`);
        const { store, queueSize, expire } = index_js_1.Configuration.getSetConfig().messages.store.acknowledged;
        const { keyMessage } = redis_keys_js_1.redisKeys.getMessageKeys(messageId);
        this.logger.debug(`Message key: ${keyMessage}, store: ${store}, queueSize: ${queueSize}, expire: ${expire}`);
        const redisClient = this.redisClient.getInstance();
        if (redisClient instanceof Error) {
            this.logger.error(`Failed to get Redis client: ${redisClient.message}`);
            this.handleError(redisClient);
            return void 0;
        }
        this.logger.debug(`Running ACKNOWLEDGE_MESSAGE script for message ${messageId}`);
        redisClient.runScript(scripts_js_1.ELuaScriptName.ACKNOWLEDGE_MESSAGE, [this.keyQueueProcessing, this.keyQueueAcknowledged, keyMessage], [
            index_js_2.EMessageProperty.STATUS,
            index_js_2.EMessagePropertyStatus.ACKNOWLEDGED,
            Number(store),
            expire,
            queueSize * -1,
        ], (err) => {
            if (err) {
                this.logger.error(`Failed to acknowledge message ${messageId}: ${err.message}`);
                this.handleError(err);
            }
            else {
                this.logger.info(`Message ${messageId} acknowledged successfully`);
                this.emit('consumer.consumeMessage.messageAcknowledged', messageId, this.queue, this.messageHandlerId, this.consumerId);
                this.logger.debug(`Emitted consumer.consumeMessage.messageAcknowledged event for message ${messageId}`);
            }
        });
    }
    unacknowledgeMessage(message, unacknowledgmentReason) {
        this.messageUnack.unacknowledgeMessage(this.consumerId, message, unacknowledgmentReason, (err, reply) => {
            if (err)
                return this.handleError(err);
            if (!reply)
                return this.handleError(new redis_smq_common_1.CallbackEmptyReplyError());
            this.handleMessageUnacknowledgementResult(unacknowledgmentReason, reply);
        });
    }
    handleMessageUnacknowledgementResult(unacknowledgmentReason, messageUnacknowledgementStatus) {
        for (const messageId in messageUnacknowledgementStatus) {
            this.logger.info(`Message ${messageId} unacknowledged successfully with reason: ${unacknowledgmentReason}`);
            this.emit('consumer.consumeMessage.messageUnacknowledged', messageId, this.queue, this.messageHandlerId, this.consumerId, unacknowledgmentReason);
            this.logger.debug(`Emitted consumer.consumeMessage.messageUnacknowledged event for message ${messageId}`);
            const unknowledgment = messageUnacknowledgementStatus[messageId];
            if (unknowledgment.action === index_js_3.EMessageUnacknowledgementAction.DEAD_LETTER) {
                this.logger.info(`Message ${messageId} moved to dead letter queue with reason: ${unknowledgment.deadLetterReason}`);
                this.emit('consumer.consumeMessage.messageDeadLettered', messageId, this.queue, this.messageHandlerId, this.consumerId, unknowledgment.deadLetterReason);
                this.logger.debug(`Emitted consumer.consumeMessage.messageDeadLettered event for message ${messageId}`);
            }
            else if (unknowledgment.action === index_js_3.EMessageUnacknowledgementAction.DELAY) {
                this.logger.info(`Message ${messageId} delayed for retry`);
                this.emit('consumer.consumeMessage.messageDelayed', messageId, this.queue, this.messageHandlerId, this.consumerId);
                this.logger.debug(`Emitted consumer.consumeMessage.messageDelayed event for message ${messageId}`);
            }
            else {
                this.logger.info(`Message ${messageId} requeued for retry`);
                this.emit('consumer.consumeMessage.messageRequeued', messageId, this.queue, this.messageHandlerId, this.consumerId);
                this.logger.debug(`Emitted consumer.consumeMessage.messageRequeued event for message ${messageId}`);
            }
        }
    }
    getConsumeMessageWorker(messageHandlerFilename) {
        this.logger.debug(`Getting consume message worker for handler file: ${messageHandlerFilename}`);
        if (!this.consumeMessageWorker) {
            this.logger.debug(`Creating new WorkerCallable for handler file: ${messageHandlerFilename}`);
            this.consumeMessageWorker = new redis_smq_common_1.WorkerCallable(messageHandlerFilename);
        }
        return this.consumeMessageWorker;
    }
    invokeMessageHandler(messageHandler, msg, cb) {
        const messageId = msg.id;
        this.logger.debug(`Invoking message handler for message ${messageId}`);
        if (typeof messageHandler === 'string') {
            this.logger.debug(`Using worker-based message handler: ${messageHandler}`);
            this.getConsumeMessageWorker(messageHandler).call(msg, (err) => {
                if (err) {
                    this.logger.error(`Worker-based message handler failed for message ${messageId}: ${err.message}`);
                }
                else {
                    this.logger.debug(`Worker-based message handler completed successfully for message ${messageId}`);
                }
                cb(err);
            });
        }
        else {
            this.logger.debug(`Using function-based message handler for message ${messageId}`);
            try {
                messageHandler(msg, (err) => {
                    if (err) {
                        this.logger.error(`Function-based message handler failed for message ${messageId}: ${err.message}`);
                    }
                    else {
                        this.logger.debug(`Function-based message handler completed successfully for message ${messageId}`);
                    }
                    cb(err);
                });
            }
            catch (err) {
                this.logger.error(`Exception in function-based message handler for message ${messageId}: ${err}`);
                cb(err instanceof Error ? err : new Error(String(err)));
            }
        }
    }
    consumeMessage(msg) {
        const messageId = msg.getId();
        this.logger.debug(`Consuming message ${messageId}`);
        let isCallbackHandled = false;
        try {
            let onConsumed = (err) => {
                if (isCallbackHandled) {
                    this.logger.debug(`Ignoring onConsumed callback for message ${messageId} as already handled`);
                    return;
                }
                isCallbackHandled = true;
                if (this.isRunning()) {
                    if (err) {
                        this.logger.error(`Error consuming message ${messageId}: ${err.message}`);
                        const unacknowledgementReason = err instanceof redis_smq_common_1.AsyncCallbackTimeoutError
                            ? index_js_3.EMessageUnacknowledgementReason.TIMEOUT
                            : index_js_3.EMessageUnacknowledgementReason.UNACKNOWLEDGED;
                        if (unacknowledgementReason ===
                            index_js_3.EMessageUnacknowledgementReason.TIMEOUT) {
                            this.logger.warn(`Consume timeout (${consumeTimeout}ms) reached for message ${messageId}`);
                        }
                        return this.unacknowledgeMessage(msg, unacknowledgementReason);
                    }
                    this.logger.debug(`Message ${messageId} consumed successfully, acknowledging`);
                    return this.acknowledgeMessage(msg);
                }
                this.logger.debug(`Ignoring onConsumed callback for message ${messageId} as consumer is not running`);
            };
            const consumeTimeout = msg.producibleMessage.getConsumeTimeout();
            if (consumeTimeout) {
                this.logger.debug(`Setting consume timeout of ${consumeTimeout}ms for message ${messageId}`);
                onConsumed = redis_smq_common_1.async.withTimeout(onConsumed, consumeTimeout);
            }
            this.logger.debug(`Transferring message ${messageId} for handler invocation`);
            this.invokeMessageHandler(this.messageHandler, msg.transfer(), onConsumed);
        }
        catch (error) {
            this.logger.error(`Exception during message consumption for message ${messageId}: ${error}`);
            isCallbackHandled = true;
            this.unacknowledgeMessage(msg, index_js_3.EMessageUnacknowledgementReason.CONSUME_ERROR);
        }
    }
    goingUp() {
        this.logger.info(`ConsumeMessage going up for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return super.goingUp().concat([
            (cb) => {
                this.logger.debug('Initializing Redis client');
                this.redisClient.init((err) => {
                    if (err) {
                        this.logger.error(`Failed to initialize Redis client: ${err.message}`);
                    }
                    else {
                        this.logger.debug('Redis client initialized successfully');
                    }
                    cb(err);
                });
            },
            (cb) => {
                this.logger.debug('Validating message handler');
                this.validateMessageHandler((err) => {
                    if (err) {
                        this.logger.error(`Message handler validation failed: ${err.message}`);
                    }
                    else {
                        this.logger.debug('Message handler validated successfully');
                    }
                    cb(err);
                });
            },
        ]);
    }
    goingDown() {
        this.logger.info(`ConsumeMessage going down for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return [
            (cb) => {
                this.logger.debug('Unacknowledging any processing messages');
                const unacknowledgementReason = index_js_3.EMessageUnacknowledgementReason.OFFLINE_MESSAGE_HANDLER;
                this.messageUnack.unacknowledgeMessagesInProcess(this.consumerId, [this.queue.queueParams], unacknowledgementReason, (err, result) => {
                    if (err) {
                        this.logger.warn(`Error during message unacknowledgement: ${err.message} (ignoring)`);
                    }
                    else if (result) {
                        this.handleMessageUnacknowledgementResult(unacknowledgementReason, result);
                    }
                    else {
                        this.logger.debug('No messages to unacknowledge during cleanup');
                    }
                    cb();
                });
            },
            (cb) => {
                if (this.consumeMessageWorker) {
                    this.logger.debug('Shutting down consume message worker');
                    this.consumeMessageWorker.shutdown((err) => {
                        if (err) {
                            this.logger.warn(`Error shutting down consume message worker: ${err.message}`);
                        }
                        else {
                            this.logger.debug('Consume message worker shut down successfully');
                        }
                        cb();
                    });
                }
                else {
                    this.logger.debug('No consume message worker to shut down');
                    cb();
                }
            },
        ].concat(super.goingDown());
    }
    handleError(err) {
        this.logger.error(`ConsumeMessage error: ${err.message}`, err);
        this.emit('consumer.consumeMessage.error', err, this.consumerId, this.queue);
        this.logger.debug(`Emitted consumer.consumeMessage.error event`);
        super.handleError(err);
    }
    handleReceivedMessage(message) {
        const messageId = message.getId();
        this.logger.debug(`Received message ${messageId}`);
        if (this.isRunning()) {
            if (message.getSetExpired()) {
                this.logger.info(`Message ${messageId} has expired, unacknowledging with TTL_EXPIRED reason`);
                this.unacknowledgeMessage(message, index_js_3.EMessageUnacknowledgementReason.TTL_EXPIRED);
            }
            else {
                this.logger.debug(`Message ${messageId} is valid, proceeding with consumption`);
                this.consumeMessage(message);
            }
        }
        else {
            this.logger.warn(`Ignoring received message ${messageId} as consumer is not running`);
        }
    }
}
exports.ConsumeMessage = ConsumeMessage;
//# sourceMappingURL=consume-message.js.map