"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageUnacknowledgement = void 0;
const redis_smq_common_1 = require("redis-smq-common");
const scripts_js_1 = require("../../../../../common/redis-client/scripts/scripts.js");
const redis_keys_js_1 = require("../../../../../common/redis-keys/redis-keys.js");
const index_js_1 = require("../../../../../config/index.js");
const index_js_2 = require("../../../../message/index.js");
const message_envelope_js_1 = require("../../../../message/message-envelope.js");
const _get_consumer_queues_js_1 = require("../../../consumer/_/_get-consumer-queues.js");
const index_js_3 = require("../../../errors/index.js");
const processing_queue_js_1 = require("./processing-queue.js");
const index_js_4 = require("./types/index.js");
class MessageUnacknowledgement {
    constructor(redisClient) {
        this.redisClient = redisClient;
        this.logger = redis_smq_common_1.logger.getLogger(index_js_1.Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
    }
    getMessageUnacknowledgementAction(message, unacknowledgedReason) {
        if (unacknowledgedReason === index_js_4.EMessageUnacknowledgementReason.TTL_EXPIRED ||
            message.getSetExpired()) {
            return {
                action: index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER,
                deadLetterReason: index_js_4.EMessageUnacknowledgementDeadLetterReason.TTL_EXPIRED,
            };
        }
        if (message.isPeriodic()) {
            return {
                action: index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER,
                deadLetterReason: index_js_4.EMessageUnacknowledgementDeadLetterReason.PERIODIC_MESSAGE,
            };
        }
        if (message.hasRetryThresholdExceeded()) {
            return {
                action: index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER,
                deadLetterReason: index_js_4.EMessageUnacknowledgementDeadLetterReason.RETRY_THRESHOLD_EXCEEDED,
            };
        }
        const delay = message.producibleMessage.getRetryDelay();
        return delay
            ? { action: index_js_4.EMessageUnacknowledgementAction.DELAY }
            : { action: index_js_4.EMessageUnacknowledgementAction.REQUEUE };
    }
    getMessageUnacknowledgementParameters(consumerId, messageOrQueue, unacknowledgementReason, done) {
        const [queue, message] = messageOrQueue instanceof message_envelope_js_1.MessageEnvelope
            ? [messageOrQueue.getDestinationQueue(), messageOrQueue]
            : [messageOrQueue, null];
        const keys = [];
        const args = [];
        const unacknowledgementStatus = {};
        args.push(JSON.stringify(queue), consumerId);
        const { keyConsumerQueues } = redis_keys_js_1.redisKeys.getConsumerKeys(consumerId);
        const { keyQueueProcessing } = redis_keys_js_1.redisKeys.getQueueConsumerKeys(queue, consumerId);
        const { keyQueueDL, keyQueueProcessingQueues, keyQueueConsumers, keyQueueProperties, keyQueueDelayed, keyQueueRequeued, } = redis_keys_js_1.redisKeys.getQueueKeys(queue, null);
        keys.push(keyQueueProcessing, keyQueueDelayed, keyQueueRequeued, keyQueueDL, keyQueueProcessingQueues, keyQueueConsumers, keyConsumerQueues, keyQueueProperties);
        if (!message) {
            keys.push('');
            args.push('', '', '', unacknowledgementReason, '');
            return done(null, { keys, args, unacknowledgementStatus });
        }
        const messageId = message.getId();
        const { keyMessage } = redis_keys_js_1.redisKeys.getMessageKeys(messageId);
        keys.push(keyMessage);
        args.push(messageId);
        const unacknowledgementAction = this.getMessageUnacknowledgementAction(message, unacknowledgementReason);
        const { action } = unacknowledgementAction;
        const messageStatus = action === index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER
            ? index_js_2.EMessagePropertyStatus.DEAD_LETTERED
            : action === index_js_4.EMessageUnacknowledgementAction.REQUEUE
                ? index_js_2.EMessagePropertyStatus.UNACK_REQUEUING
                : index_js_2.EMessagePropertyStatus.UNACK_DELAYING;
        args.push(action, action === index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER
            ? unacknowledgementAction.deadLetterReason
            : '', unacknowledgementReason, messageStatus);
        unacknowledgementStatus[messageId] = unacknowledgementAction;
        done(null, {
            args,
            keys,
            unacknowledgementStatus,
        });
    }
    executeMessageUnacknowledgement(keys, argv, messageHandlingStatus, cb) {
        (0, redis_smq_common_1.withRedisClient)(this.redisClient, (client, cb) => {
            const { store, expire, queueSize } = index_js_1.Configuration.getSetConfig().messages.store.deadLettered;
            const execKeys = [...keys];
            const execArgs = [
                index_js_4.EMessageUnacknowledgementAction.DELAY,
                index_js_4.EMessageUnacknowledgementAction.REQUEUE,
                index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER,
                Number(store),
                expire,
                queueSize * -1,
                index_js_2.EMessageProperty.STATUS,
                index_js_4.EMessageUnacknowledgementReason.OFFLINE_CONSUMER,
                index_js_4.EMessageUnacknowledgementReason.OFFLINE_MESSAGE_HANDLER,
                ...argv,
            ];
            client.runScript(scripts_js_1.ELuaScriptName.HANDLE_PROCESSING_QUEUE, execKeys, execArgs, (err, reply) => {
                if (err) {
                    return cb(err);
                }
                if (reply !== 'OK') {
                    return cb(new index_js_3.ConsumerError(reply ? String(reply) : undefined));
                }
                Object.keys(messageHandlingStatus).forEach((messageId) => {
                    const action = messageHandlingStatus[messageId].action ===
                        index_js_4.EMessageUnacknowledgementAction.DEAD_LETTER
                        ? 'dead-lettered'
                        : 'unacknowledged';
                    this.logger.debug(`Message ID ${messageId} has been ${action}.`);
                });
                cb(null, messageHandlingStatus);
            });
        }, cb);
    }
    unacknowledgeMessage(consumerId, msg, unacknowledgementReason, cb) {
        this.getMessageUnacknowledgementParameters(consumerId, msg, unacknowledgementReason, (err, reply) => {
            if (err)
                return cb(err);
            if (!reply)
                return cb(new redis_smq_common_1.CallbackEmptyReplyError());
            const { keys, args, unacknowledgementStatus } = reply;
            this.executeMessageUnacknowledgement(keys, args, unacknowledgementStatus !== null && unacknowledgementStatus !== void 0 ? unacknowledgementStatus : {}, cb);
        });
    }
    unacknowledgeMessagesInProcess(consumerId, queues, unacknowledgementReason, cb) {
        const keys = [];
        const args = [];
        const messageHandlingStatus = {};
        (0, redis_smq_common_1.withRedisClient)(this.redisClient, (client, cb) => {
            redis_smq_common_1.async.waterfall([
                (next) => {
                    if (queues === null)
                        (0, _get_consumer_queues_js_1._getConsumerQueues)(client, consumerId, next);
                    else
                        next(null, queues);
                },
                (queueParams, next) => {
                    if (!queueParams.length) {
                        return next();
                    }
                    redis_smq_common_1.async.eachOf(queueParams, (queue, _, done) => {
                        const { keyQueueProcessing } = redis_keys_js_1.redisKeys.getQueueConsumerKeys(queue, consumerId);
                        processing_queue_js_1.processingQueue.fetchMessageFromProcessingQueue(this.redisClient, keyQueueProcessing, (err, message) => {
                            if (err)
                                return done(err);
                            redis_smq_common_1.async.withCallback((cb) => this.getMessageUnacknowledgementParameters(consumerId, message !== null && message !== void 0 ? message : queue, unacknowledgementReason, cb), (reply, cb) => {
                                keys.push(...reply.keys);
                                args.push(...reply.args);
                                Object.assign(messageHandlingStatus, reply.unacknowledgementStatus);
                                cb();
                            }, done);
                        });
                    }, next);
                },
            ], (err) => {
                if (err) {
                    return cb(err);
                }
                if (!keys.length) {
                    return cb(null, {});
                }
                this.executeMessageUnacknowledgement(keys, args, messageHandlingStatus, cb);
            });
        }, cb);
    }
}
exports.MessageUnacknowledgement = MessageUnacknowledgement;
//# sourceMappingURL=message-unacknowledgement.js.map