"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageHandler = void 0;
const path_1 = __importDefault(require("path"));
const redis_smq_common_1 = require("redis-smq-common");
const redis_client_js_1 = require("../../../../common/redis-client/redis-client.js");
const scripts_js_1 = require("../../../../common/redis-client/scripts/scripts.js");
const redis_keys_js_1 = require("../../../../common/redis-keys/redis-keys.js");
const index_js_1 = require("../../../../config/index.js");
const _from_message_js_1 = require("../../../message/_/_from-message.js");
const index_js_2 = require("../../../message/index.js");
const consume_message_js_1 = require("./consume-message/consume-message.js");
const dequeue_message_js_1 = require("./dequeue-message/dequeue-message.js");
const even_bus_publisher_js_1 = require("./even-bus-publisher.js");
class MessageHandler extends redis_smq_common_1.Runnable {
    constructor(consumer, redisClient, handlerParams, autoDequeue = true, eventBus) {
        super();
        this.workerResourceGroup = null;
        this.onMessageReceived = (messageId) => {
            this.logger.debug(`Message received event for message ${messageId}, processing`);
            this.processMessage(messageId);
        };
        this.onMessageUnacknowledged = (messageId) => {
            this.logger.debug(`Message unacknowledged event for message ${messageId}, requesting next message`);
            this.next();
        };
        this.onMessageAcknowledged = (messageId) => {
            this.logger.debug(`Message acknowledged event for message ${messageId}, requesting next message`);
            this.next();
        };
        this.onMessageNext = () => {
            this.logger.debug('Next message event received, requesting next message');
            this.next();
        };
        this.onError = (err) => {
            if (this.isRunning()) {
                this.logger.error(`Error event received while running: ${err.message}`);
                this.handleError(err);
            }
            else {
                this.logger.debug(`Ignoring error event received during shutdown: ${err.message}`);
            }
        };
        this.registerSystemEvents = () => {
            this.logger.debug('Registering system event handlers');
            this.dequeueMessage.on('consumer.dequeueMessage.messageReceived', this.onMessageReceived);
            this.dequeueMessage.on('consumer.dequeueMessage.nextMessage', this.onMessageNext);
            this.consumeMessage.on('consumer.consumeMessage.messageUnacknowledged', this.onMessageUnacknowledged);
            this.consumeMessage.on('consumer.consumeMessage.messageAcknowledged', this.onMessageAcknowledged);
            this.logger.debug('All system event handlers registered successfully');
        };
        this.setUpConsumerWorkers = (cb) => {
            this.logger.debug('Setting up consumer workers');
            const config = index_js_1.Configuration.getSetConfig();
            const { keyQueueWorkersLock } = redis_keys_js_1.redisKeys.getQueueKeys(this.queue.queueParams, this.queue.groupId);
            this.logger.debug(`Queue workers lock key: ${keyQueueWorkersLock}`);
            const redisClient = this.redisClient.getInstance();
            if (redisClient instanceof Error) {
                this.logger.error(`Failed to get Redis client: ${redisClient.message}`);
                cb(redisClient);
                return void 0;
            }
            this.logger.debug('Creating WorkerResourceGroup');
            this.workerResourceGroup = new redis_smq_common_1.WorkerResourceGroup(redisClient, this.logger, keyQueueWorkersLock);
            this.logger.debug('Setting up error handler for WorkerResourceGroup');
            this.workerResourceGroup.on('workerResourceGroup.error', (err) => {
                this.logger.error(`WorkerResourceGroup error: ${err.message}`);
                this.handleError(err);
            });
            const workersDir = path_1.default.resolve(redis_smq_common_1.env.getCurrentDir(), '../../workers');
            this.logger.debug(`Loading workers from directory: ${workersDir}`);
            this.workerResourceGroup.loadFromDir(workersDir, { config, queueParsedParams: this.queue }, (err) => {
                var _a;
                if (err) {
                    this.logger.error(`Failed to load workers from directory: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.debug('Workers loaded successfully, starting WorkerResourceGroup');
                    (_a = this.workerResourceGroup) === null || _a === void 0 ? void 0 : _a.run((err) => {
                        if (err) {
                            this.logger.error(`Failed to start WorkerResourceGroup: ${err.message}`);
                            this.handleError(err);
                        }
                        else {
                            this.logger.debug('WorkerResourceGroup started successfully');
                        }
                    });
                    this.logger.debug('Consumer workers setup completed');
                    cb();
                }
            });
        };
        this.shutDownConsumerWorkers = (cb) => {
            if (this.workerResourceGroup) {
                this.logger.debug('Shutting down consumer workers');
                this.workerResourceGroup.shutdown((err) => {
                    if (err) {
                        this.logger.warn(`Error during WorkerResourceGroup shutdown: ${err.message} (ignoring)`);
                    }
                    this.logger.debug('Consumer workers shut down successfully');
                    this.workerResourceGroup = null;
                    cb();
                });
            }
            else {
                this.logger.debug('No consumer workers to shut down');
                cb();
            }
        };
        const { queue, messageHandler } = handlerParams;
        this.consumer = consumer;
        this.consumerId = consumer.getId();
        this.queue = queue;
        this.messageHandler = messageHandler;
        this.logger = redis_smq_common_1.logger.getLogger(index_js_1.Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
        this.logger.info(`Initializing MessageHandler for consumer ${this.consumerId}, queue ${JSON.stringify(this.queue)}`);
        this.logger.debug(`autoDequeue: ${autoDequeue}, messageHandler type: ${typeof messageHandler}`);
        this.autoDequeue = autoDequeue;
        this.redisClient = redisClient;
        this.eventBus = eventBus;
        if (this.eventBus) {
            this.logger.debug('Event bus provided, setting up event bus publisher');
            (0, even_bus_publisher_js_1.evenBusPublisher)(this, this.eventBus, this.logger);
        }
        else {
            this.logger.debug('No event bus provided, skipping event bus publisher setup');
        }
        this.logger.debug('Initializing DequeueMessage instance');
        this.dequeueMessage = this.initDequeueMessageInstance();
        this.logger.debug('Initializing ConsumeMessage instance');
        this.consumeMessage = this.initConsumeMessageInstance();
        this.logger.debug('Registering system events');
        this.registerSystemEvents();
        this.logger.info(`MessageHandler initialized for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
    }
    initDequeueMessageInstance() {
        this.logger.debug('Creating new DequeueMessage instance');
        const instance = new dequeue_message_js_1.DequeueMessage(new redis_client_js_1.RedisClient(), this.queue, this.consumer, this.eventBus);
        this.logger.debug('Setting up error handler for DequeueMessage instance');
        instance.on('consumer.dequeueMessage.error', this.onError);
        this.logger.debug('DequeueMessage instance created successfully');
        return instance;
    }
    initConsumeMessageInstance() {
        this.logger.debug('Creating new ConsumeMessage instance');
        const instance = new consume_message_js_1.ConsumeMessage(this.redisClient, this.consumer, this.queue, this.getId(), this.messageHandler, this.eventBus);
        this.logger.debug('Setting up error handler for ConsumeMessage instance');
        instance.on('consumer.consumeMessage.error', this.onError);
        this.logger.debug('ConsumeMessage instance created successfully');
        return instance;
    }
    getLogger() {
        return this.logger;
    }
    handleError(err) {
        if (this.isRunning()) {
            this.logger.error(`MessageHandler error: ${err.message}`, err);
            this.logger.debug(`Emitting consumer.messageHandler.error event for consumer ${this.consumerId}`);
            this.emit('consumer.messageHandler.error', err, this.consumerId, this.queue);
        }
        else {
            this.logger.debug(`Error occurred while not running: ${err.message} (ignoring)`);
        }
        super.handleError(err);
    }
    goingUp() {
        this.logger.info(`MessageHandler going up for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return super.goingUp().concat([
            (cb) => {
                this.logger.debug('Starting DequeueMessage instance');
                this.dequeueMessage.run((err) => {
                    if (err) {
                        this.logger.error(`Failed to start DequeueMessage: ${err.message}`);
                    }
                    else {
                        this.logger.debug('DequeueMessage started successfully');
                    }
                    cb(err);
                });
            },
            (cb) => {
                this.logger.debug('Starting ConsumeMessage instance');
                this.consumeMessage.run((err) => {
                    if (err) {
                        this.logger.error(`Failed to start ConsumeMessage: ${err.message}`);
                    }
                    else {
                        this.logger.debug('ConsumeMessage started successfully');
                    }
                    cb(err);
                });
            },
            (cb) => {
                if (this.autoDequeue) {
                    this.logger.debug('Auto-dequeue enabled, initiating first dequeue');
                    this.dequeue();
                }
                else {
                    this.logger.debug('Auto-dequeue disabled, skipping initial dequeue');
                }
                cb();
            },
            this.setUpConsumerWorkers,
        ]);
    }
    goingDown() {
        this.logger.info(`MessageHandler going down for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return [
            this.shutDownConsumerWorkers,
            (cb) => {
                this.logger.debug('Shutting down DequeueMessage instance');
                this.dequeueMessage.shutdown((err) => {
                    if (err) {
                        this.logger.warn(`Error during DequeueMessage shutdown: ${err.message} (ignoring)`);
                    }
                    else {
                        this.logger.debug('DequeueMessage shut down successfully');
                    }
                    cb();
                });
            },
            (cb) => {
                this.logger.debug('Shutting down ConsumeMessage instance');
                this.consumeMessage.shutdown((err) => {
                    if (err) {
                        this.logger.warn(`Error during ConsumeMessage shutdown: ${err.message} (ignoring)`);
                    }
                    else {
                        this.logger.debug('ConsumeMessage shut down successfully');
                    }
                    cb();
                });
            },
        ].concat(super.goingDown());
    }
    processMessage(messageId) {
        if (!this.isRunning()) {
            this.logger.debug(`Ignoring processMessage call for message ${messageId} as handler is not running`);
            return void 0;
        }
        this.logger.debug(`Processing message ${messageId}`);
        const { keyMessage } = redis_keys_js_1.redisKeys.getMessageKeys(messageId);
        this.logger.debug(`Message key: ${keyMessage}`);
        const keys = [keyMessage];
        const argv = [
            index_js_2.EMessageProperty.STATUS,
            index_js_2.EMessageProperty.STATE,
            index_js_2.EMessageProperty.MESSAGE,
            index_js_2.EMessagePropertyStatus.PROCESSING,
        ];
        const redisClient = this.redisClient.getInstance();
        if (redisClient instanceof Error) {
            this.logger.error(`Failed to get Redis client: ${redisClient.message}`);
            this.handleError(redisClient);
            return void 0;
        }
        this.logger.debug(`Running FETCH_MESSAGE_FOR_PROCESSING script for message ${messageId}`);
        redisClient.runScript(scripts_js_1.ELuaScriptName.FETCH_MESSAGE_FOR_PROCESSING, keys, argv, (err, reply) => {
            if (err) {
                this.logger.error(`Failed to fetch message ${messageId} for processing: ${err.message}`);
                return this.handleError(err);
            }
            if (!reply) {
                this.logger.error(`Empty reply when fetching message ${messageId}`);
                return this.handleError(new redis_smq_common_1.CallbackEmptyReplyError());
            }
            if (!Array.isArray(reply)) {
                this.logger.error(`Invalid reply type when fetching message ${messageId}`);
                return this.handleError(new redis_smq_common_1.CallbackInvalidReplyError());
            }
            const [state, msg] = reply;
            this.logger.debug(`Message ${messageId} fetched successfully, state: ${state}`);
            const message = (0, _from_message_js_1._fromMessage)(msg, index_js_2.EMessagePropertyStatus.PROCESSING, state);
            this.logger.debug(`Passing message ${messageId} to ConsumeMessage for handling`);
            this.consumeMessage.handleReceivedMessage(message);
        });
    }
    next() {
        this.logger.debug('Requesting next message');
        this.dequeue();
    }
    dequeue() {
        if (this.isRunning()) {
            this.logger.debug('Dequeuing message from queue');
            this.dequeueMessage.dequeue();
        }
        else {
            this.logger.debug('Ignoring dequeue call as handler is not running');
        }
    }
    getQueue() {
        this.logger.debug(`Getting queue parameters: ${JSON.stringify(this.queue)}`);
        return this.queue;
    }
}
exports.MessageHandler = MessageHandler;
//# sourceMappingURL=message-handler.js.map