"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const redis_smq_common_1 = require("redis-smq-common");
const scripts_js_1 = require("../../../common/redis-client/scripts/scripts.js");
const redis_keys_js_1 = require("../../../common/redis-keys/redis-keys.js");
const _get_message_js_1 = require("../../message/_/_get-message.js");
const index_js_1 = require("../../message/index.js");
const index_js_2 = require("../../queue/index.js");
const worker_js_1 = require("./worker.js");
class DelayUnacknowledgedWorker extends worker_js_1.Worker {
    constructor() {
        super(...arguments);
        this.work = (cb) => {
            this.logger.debug('Starting DelayUnacknowledgedWorker work cycle');
            const { keyQueueDelayed, keyQueueScheduled, keyQueueProperties, keyQueueMessages, } = redis_keys_js_1.redisKeys.getQueueKeys(this.queueParsedParams.queueParams, this.queueParsedParams.groupId);
            this.logger.debug(`Working with queue: ${this.queueParsedParams.queueParams.name}, group: ${this.queueParsedParams.groupId || 'none'}`);
            const redisClient = this.redisClient.getInstance();
            if (redisClient instanceof Error) {
                this.logger.error('Failed to get Redis client instance', redisClient);
                cb(redisClient);
                return void 0;
            }
            this.logger.debug(`Fetching delayed messages from ${keyQueueDelayed}`);
            redisClient.lrange(keyQueueDelayed, 0, 9, (err, reply) => {
                if (err) {
                    this.logger.error('Error fetching delayed messages', err);
                    cb(err);
                }
                else {
                    const messageIds = reply !== null && reply !== void 0 ? reply : [];
                    this.logger.debug(`Found ${messageIds.length} delayed messages to process`);
                    if (messageIds.length) {
                        const keys = [];
                        const args = [
                            index_js_2.EQueueProperty.QUEUE_TYPE,
                            index_js_2.EQueueProperty.MESSAGES_COUNT,
                            index_js_1.EMessageProperty.MESSAGE,
                            index_js_1.EMessageProperty.STATUS,
                            index_js_1.EMessagePropertyStatus.SCHEDULED,
                            index_js_1.EMessageProperty.STATE,
                            '1',
                        ];
                        this.logger.debug('Processing delayed messages batch');
                        redis_smq_common_1.async.each(messageIds, (messageId, _, done) => {
                            this.logger.debug(`Processing delayed message: ${messageId}`);
                            (0, _get_message_js_1._getMessage)(redisClient, messageId, (err, message) => {
                                if (err) {
                                    this.logger.error(`Error retrieving message ${messageId}`, err);
                                    done(err);
                                }
                                else if (!message) {
                                    this.logger.error(`Message ${messageId} not found`);
                                    cb(new redis_smq_common_1.CallbackEmptyReplyError());
                                }
                                else {
                                    const messageId = message.getId();
                                    this.logger.debug(`Preparing message ${messageId} for scheduling`);
                                    const { keyMessage } = redis_keys_js_1.redisKeys.getMessageKeys(messageId);
                                    keys.push(keyQueueMessages, keyQueueProperties, keyMessage, keyQueueScheduled, keyQueueDelayed);
                                    args.push(messageId, '');
                                    const delay = message.producibleMessage.getRetryDelay();
                                    this.logger.debug(`Message ${messageId} retry delay: ${delay}ms`);
                                    const messageState = message.getMessageState();
                                    const previousAttempts = messageState.getAttempts();
                                    messageState.incrAttempts();
                                    messageState.setNextRetryDelay(delay);
                                    this.logger.debug(`Message ${messageId} attempts incremented from ${previousAttempts} to ${messageState.getAttempts()}`);
                                    const timestamp = message.getNextScheduledTimestamp();
                                    this.logger.debug(`Message ${messageId} scheduled for timestamp: ${timestamp}`);
                                    args.push(timestamp, JSON.stringify(messageState));
                                    done();
                                }
                            });
                        }, (err) => {
                            if (err) {
                                this.logger.error('Error processing delayed messages batch', err);
                                cb(err);
                            }
                            else {
                                this.logger.debug('Running SCHEDULE_MESSAGE script for delayed messages');
                                redisClient.runScript(scripts_js_1.ELuaScriptName.SCHEDULE_MESSAGE, keys, args, (err, reply) => {
                                    if (err) {
                                        this.logger.error('Error running SCHEDULE_MESSAGE script', err);
                                        cb(err);
                                    }
                                    else if (!reply) {
                                        this.logger.error('Empty reply from SCHEDULE_MESSAGE script');
                                        cb(new redis_smq_common_1.CallbackEmptyReplyError());
                                    }
                                    else if (reply !== 'OK') {
                                        this.logger.error(`Unexpected reply from SCHEDULE_MESSAGE script: ${reply}`);
                                        cb(new redis_smq_common_1.PanicError(String(reply)));
                                    }
                                    else {
                                        this.logger.debug('Successfully scheduled delayed messages');
                                        cb();
                                    }
                                });
                            }
                        });
                    }
                    else {
                        this.logger.debug('No delayed messages to process');
                        cb();
                    }
                }
            });
        };
    }
}
exports.default = (payload) => new DelayUnacknowledgedWorker(payload);
//# sourceMappingURL=delay-unacknowledged.worker.js.map