"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const redis_smq_common_1 = require("redis-smq-common");
const scripts_js_1 = require("../../../common/redis-client/scripts/scripts.js");
const redis_keys_js_1 = require("../../../common/redis-keys/redis-keys.js");
const _get_message_js_1 = require("../../message/_/_get-message.js");
const index_js_1 = require("../../message/index.js");
const index_js_2 = require("../../queue/index.js");
const worker_js_1 = require("./worker.js");
class RequeueUnacknowledgedWorker extends worker_js_1.Worker {
    constructor() {
        super(...arguments);
        this.work = (cb) => {
            this.logger.debug('Starting requeue unacknowledged messages work cycle');
            const { keyQueueRequeued, keyQueueProperties } = redis_keys_js_1.redisKeys.getQueueKeys(this.queueParsedParams.queueParams, this.queueParsedParams.groupId);
            this.logger.debug(`Queue: ${this.queueParsedParams.queueParams.ns}:${this.queueParsedParams.queueParams.name}, GroupId: ${this.queueParsedParams.groupId || 'none'}`);
            this.logger.debug(`Using requeued queue key: ${keyQueueRequeued}`);
            const redisClient = this.redisClient.getInstance();
            if (redisClient instanceof Error) {
                this.logger.error('Failed to get Redis client instance', redisClient);
                cb(redisClient);
                return void 0;
            }
            redisClient.lrange(keyQueueRequeued, 0, 99, (err, reply) => {
                if (err) {
                    this.logger.error('Error retrieving messages from requeued queue', err);
                    cb(err);
                }
                else {
                    const messageIds = reply !== null && reply !== void 0 ? reply : [];
                    const messageCount = messageIds.length;
                    this.logger.debug(`Found ${messageCount} messages to requeue`);
                    if (messageCount) {
                        const keys = [keyQueueRequeued, keyQueueProperties];
                        const argv = [
                            index_js_2.EQueueProperty.QUEUE_TYPE,
                            index_js_2.EQueueType.PRIORITY_QUEUE,
                            index_js_2.EQueueType.LIFO_QUEUE,
                            index_js_2.EQueueType.FIFO_QUEUE,
                            index_js_1.EMessageProperty.STATUS,
                            index_js_1.EMessagePropertyStatus.PENDING,
                            index_js_1.EMessageProperty.STATE,
                        ];
                        this.logger.debug('Processing messages for requeue operation');
                        redis_smq_common_1.async.eachOf(messageIds, (messageId, index, done) => {
                            this.logger.debug(`Processing message ${messageId} (${index + 1}/${messageCount})`);
                            (0, _get_message_js_1._getMessage)(redisClient, messageId, (err, message) => {
                                var _a;
                                if (err) {
                                    this.logger.error(`Error retrieving message ${messageId}`, err);
                                    done(err);
                                }
                                else if (!message) {
                                    this.logger.error(`Message ${messageId} not found`);
                                    cb(new redis_smq_common_1.CallbackEmptyReplyError());
                                }
                                else {
                                    const messageId = message.getId();
                                    const messageState = message.getMessageState();
                                    const destinationQueue = message.getDestinationQueue();
                                    const consumerGroupId = message.getConsumerGroupId();
                                    this.logger.debug(`Preparing to requeue message ${messageId} to ${destinationQueue.ns}:${destinationQueue.name}, consumer group: ${consumerGroupId || 'none'}`);
                                    const { keyQueuePriorityPending, keyQueuePending } = redis_keys_js_1.redisKeys.getQueueKeys(destinationQueue, consumerGroupId);
                                    const { keyMessage } = redis_keys_js_1.redisKeys.getMessageKeys(messageId);
                                    keys.push(keyQueuePriorityPending, keyQueuePending, keyMessage);
                                    const previousAttempts = messageState.getAttempts();
                                    messageState.incrAttempts();
                                    const newAttempts = messageState.getAttempts();
                                    this.logger.debug(`Incremented message ${messageId} attempts from ${previousAttempts} to ${newAttempts}`);
                                    const messagePriority = (_a = message.producibleMessage.getPriority()) !== null && _a !== void 0 ? _a : '';
                                    argv.push(messageId, messagePriority, JSON.stringify(messageState));
                                    done();
                                }
                            });
                        }, (err) => {
                            if (err) {
                                this.logger.error('Error during message processing for requeue', err);
                                cb(err);
                            }
                            else {
                                this.logger.debug(`Executing REQUEUE_MESSAGE script with ${messageCount} messages`);
                                redisClient.runScript(scripts_js_1.ELuaScriptName.REQUEUE_MESSAGE, keys, argv, (err) => {
                                    if (err) {
                                        this.logger.error('Error executing requeue script', err);
                                        cb(err);
                                    }
                                    else {
                                        this.logger.info(`Successfully requeued ${messageCount} messages`);
                                        cb();
                                    }
                                });
                            }
                        });
                    }
                    else {
                        this.logger.debug('No messages to requeue, work cycle complete');
                        cb();
                    }
                }
            });
        };
    }
}
exports.default = (payload) => new RequeueUnacknowledgedWorker(payload);
//# sourceMappingURL=requeue-unacknowledged.worker.js.map