"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueueMessages = void 0;
const redis_smq_common_1 = require("redis-smq-common");
const redis_client_js_1 = require("../../../common/redis-client/redis-client.js");
const _get_consumer_groups_js_1 = require("../../consumer-groups/_/_get-consumer-groups.js");
const index_js_1 = require("../../message/index.js");
const _get_queue_properties_js_1 = require("../../queue/_/_get-queue-properties.js");
const _parse_queue_params_js_1 = require("../../queue/_/_parse-queue-params.js");
const index_js_2 = require("../../queue/index.js");
const queue_messages_manager_abstract_js_1 = require("../queue-messages-manager/queue-messages-manager-abstract.js");
const queue_messages_storage_set_js_1 = require("../queue-messages-storage/queue-messages-storage-set.js");
const queue_acknowledged_messages_js_1 = require("./queue-acknowledged-messages.js");
const queue_dead_lettered_messages_js_1 = require("./queue-dead-lettered-messages.js");
const sequential_queue_pending_messages_js_1 = require("./queue-pending-messages/sequential-queue-pending-messages.js");
const priority_queue_pending_messages_js_1 = require("./queue-pending-messages/priority-queue-pending-messages.js");
const queue_scheduled_messages_js_1 = require("./queue-scheduled-messages.js");
class QueueMessages extends queue_messages_manager_abstract_js_1.QueueMessagesManagerAbstract {
    constructor() {
        const redisClient = new redis_client_js_1.RedisClient();
        super(redisClient, new queue_messages_storage_set_js_1.QueueMessagesStorageSet(redisClient), new index_js_1.Message(), 'keyQueueMessages');
        this.priorityQueueMessages = new priority_queue_pending_messages_js_1.PriorityQueuePendingMessages();
        this.queuePendingMessages = new sequential_queue_pending_messages_js_1.SequentialQueuePendingMessages();
        this.queueDeadLetteredMessages = new queue_dead_lettered_messages_js_1.QueueDeadLetteredMessages();
        this.queueScheduledMessages = new queue_scheduled_messages_js_1.QueueScheduledMessages();
        this.queueAcknowledgedMessages = new queue_acknowledged_messages_js_1.QueueAcknowledgedMessages();
        this.logger.debug('QueueMessages initialized');
    }
    countMessagesByStatus(queue, cb) {
        const queueParams = (0, _parse_queue_params_js_1._parseQueueParams)(queue);
        if (queueParams instanceof Error) {
            return cb(queueParams);
        }
        (0, redis_smq_common_1.withRedisClient)(this.redisClient, (client, cb) => this.executeCountingPipeline(client, queueParams, queue, cb), cb);
    }
    executeCountingPipeline(client, queueParams, queue, cb) {
        const count = {
            pending: 0,
            acknowledged: 0,
            scheduled: 0,
            deadLettered: 0,
        };
        redis_smq_common_1.async.waterfall([
            (next) => this.getQueueProperties(client, queueParams, next),
            (properties, next) => this.countPendingMessages(client, properties, queue, queueParams, next),
            (pendingCount, next) => {
                count.pending = pendingCount;
                this.queueDeadLetteredMessages.countMessages(queue, next);
            },
            (deadLettered, next) => {
                count.deadLettered = deadLettered;
                this.queueAcknowledgedMessages.countMessages(queue, next);
            },
            (acknowledged, next) => {
                count.acknowledged = acknowledged;
                this.queueScheduledMessages.countMessages(queue, next);
            },
        ], (err, scheduled) => {
            if (err) {
                return cb(err);
            }
            count.scheduled = Number(scheduled);
            return cb(null, count);
        });
    }
    getQueueProperties(client, queueParams, cb) {
        (0, _get_queue_properties_js_1._getQueueProperties)(client, queueParams, (err, properties) => {
            if (err) {
                return cb(err);
            }
            if (!properties) {
                return cb(new redis_smq_common_1.CallbackEmptyReplyError());
            }
            return cb(null, properties);
        });
    }
    countPendingMessages(client, properties, queue, queueParams, cb) {
        const { queueType, deliveryModel } = properties;
        const countPendingForGroup = (groupId, callback) => {
            if (queueType === index_js_2.EQueueType.PRIORITY_QUEUE) {
                this.priorityQueueMessages.countMessages({ queue, groupId }, callback);
            }
            else {
                this.queuePendingMessages.countMessages({ queue, groupId }, callback);
            }
        };
        if (deliveryModel === index_js_2.EQueueDeliveryModel.PUB_SUB) {
            this.countPendingByGroupId(client, queueParams, countPendingForGroup, cb);
        }
        else {
            countPendingForGroup(null, cb);
        }
    }
    countPendingByGroupId(client, queueParams, countPendingForGroup, cb) {
        (0, _get_consumer_groups_js_1._getConsumerGroups)(client, queueParams, (err, groups) => {
            if (err) {
                return cb(err);
            }
            const pendingCounts = {};
            redis_smq_common_1.async.each(groups !== null && groups !== void 0 ? groups : [], (groupId, _, next) => {
                countPendingForGroup(groupId, (err, cnt) => {
                    if (err) {
                        return next(err);
                    }
                    pendingCounts[groupId] = Number(cnt);
                    return next();
                });
            }, (err) => cb(err, pendingCounts));
        });
    }
    shutdown(cb) {
        redis_smq_common_1.async.series([
            (next) => this.queuePendingMessages.shutdown(next),
            (next) => this.priorityQueueMessages.shutdown(next),
            (next) => this.queueScheduledMessages.shutdown(next),
            (next) => this.queueAcknowledgedMessages.shutdown(next),
            (next) => this.queueDeadLetteredMessages.shutdown(next),
            (next) => super.shutdown(next),
        ], (err) => cb(err));
    }
}
exports.QueueMessages = QueueMessages;
//# sourceMappingURL=queue-messages.js.map