import { logger, Runnable, } from 'redis-smq-common';
import { EventBus } from '../../../common/index.js';
import { RedisClient } from '../../../common/redis-client/redis-client.js';
import { Configuration } from '../../../config/index.js';
import { _parseQueueExtendedParams } from '../../queue/_/_parse-queue-extended-params.js';
import { ConsumerHeartbeat } from '../consumer-heartbeat/consumer-heartbeat.js';
import { MessageHandlerRunner } from '../message-handler-runner/message-handler-runner.js';
import { MultiplexedMessageHandlerRunner } from '../message-handler-runner/multiplexed-message-handler-runner.js';
import { eventBusPublisher } from './event-bus-publisher.js';
export class Consumer extends Runnable {
    messageHandlerRunner;
    logger;
    redisClient;
    eventBus = null;
    heartbeat = null;
    constructor(enableMultiplexing) {
        super();
        const config = Configuration.getSetConfig();
        this.logger = logger.getLogger(config.logger, this.constructor.name.toLowerCase());
        this.logger.info(`Initializing consumer${enableMultiplexing ? ' with multiplexing enabled' : ''}`);
        this.redisClient = new RedisClient();
        this.redisClient.on('error', this.onRedisError);
        this.logger.debug('Redis client initialized');
        if (Configuration.getSetConfig().eventBus.enabled) {
            this.logger.debug('Event bus is enabled, initializing');
            this.eventBus = new EventBus();
            this.eventBus.on('error', this.onRedisError);
            eventBusPublisher(this, this.eventBus, this.logger);
            this.logger.debug('Event bus publisher configured');
        }
        else {
            this.logger.debug('Event bus is disabled');
        }
        this.logger.debug(`Creating ${enableMultiplexing ? 'multiplexed' : 'standard'} message handler runner`);
        this.messageHandlerRunner = enableMultiplexing
            ? new MultiplexedMessageHandlerRunner(this, this.redisClient, this.eventBus)
            : new MessageHandlerRunner(this, this.redisClient, this.eventBus);
        this.messageHandlerRunner.on('consumer.messageHandlerRunner.error', (err) => {
            this.logger.error(`Message handler runner error: ${err.message}`);
            this.handleError(err);
        });
        this.logger.info(`Consumer initialized with ID: ${this.id}`);
    }
    onRedisError = (error) => {
        this.logger.error(`Redis client error: ${error.message}`);
        this.handleError(error);
    };
    setUpHeartbeat = (cb) => {
        this.logger.debug('Setting up consumer heartbeat');
        this.heartbeat = new ConsumerHeartbeat(this, this.redisClient, this.eventBus ?? null);
        this.heartbeat.on('consumerHeartbeat.error', (err) => {
            this.logger.error(`Heartbeat error: ${err.message}`);
            this.handleError(err);
        });
        this.logger.debug('Starting heartbeat');
        this.heartbeat.run((err) => {
            if (err) {
                this.logger.error(`Failed to start heartbeat: ${err.message}`);
                cb(err);
            }
            else {
                this.logger.debug('Heartbeat started successfully');
                cb();
            }
        });
    };
    shutDownHeartbeat = (cb) => {
        if (this.heartbeat) {
            this.logger.debug('Shutting down heartbeat');
            this.heartbeat.shutdown((err) => {
                if (err) {
                    this.logger.warn(`Error during heartbeat shutdown: ${err.message}`);
                }
                this.logger.debug('Heartbeat shut down');
                this.heartbeat = null;
                cb();
            });
        }
        else {
            this.logger.debug('No heartbeat to shut down');
            cb();
        }
    };
    runMessageHandlers = (cb) => {
        this.logger.debug('Starting message handlers');
        this.messageHandlerRunner.run((err) => {
            if (err) {
                this.logger.error(`Failed to start message handlers: ${err.message}`);
                cb(err);
            }
            else {
                this.logger.debug('Message handlers started successfully');
                cb();
            }
        });
    };
    shutdownMessageHandlers = (cb) => {
        this.logger.debug('Shutting down message handlers');
        this.messageHandlerRunner.shutdown((err) => {
            if (err) {
                this.logger.warn(`Error during message handlers shutdown: ${err.message}`);
            }
            this.logger.debug('Message handlers shut down');
            cb();
        });
    };
    shutDownRedisClient = (cb) => {
        this.logger.debug('Shutting down Redis client');
        this.redisClient.shutdown((err) => {
            if (err) {
                this.logger.warn(`Error during Redis client shutdown: ${err.message}`);
            }
            this.logger.debug('Redis client shut down');
            cb();
        });
    };
    goingUp() {
        this.logger.info('Consumer going up');
        return super.goingUp().concat([
            (cb) => {
                if (this.eventBus) {
                    this.logger.debug('Initializing event bus');
                    this.eventBus.init((err) => {
                        if (err) {
                            this.logger.error(`Failed to initialize event bus: ${err.message}`);
                        }
                        else {
                            this.logger.debug('Event bus initialized');
                        }
                        cb(err);
                    });
                }
                else {
                    this.logger.debug('Skipping event bus initialization (disabled)');
                    cb();
                }
            },
            (cb) => {
                this.logger.debug(`Emitting consumer.goingUp event for consumer ${this.id}`);
                this.emit('consumer.goingUp', this.id);
                cb();
            },
            this.redisClient.init,
            this.setUpHeartbeat,
            this.runMessageHandlers,
        ]);
    }
    goingDown() {
        this.logger.info('Consumer going down');
        this.logger.debug(`Emitting consumer.goingDown event for consumer ${this.id}`);
        this.emit('consumer.goingDown', this.id);
        return [
            this.shutdownMessageHandlers,
            this.shutDownHeartbeat,
            this.shutDownRedisClient,
        ].concat(super.goingDown());
    }
    up(cb) {
        this.logger.info('Consumer is up');
        super.up(() => {
            this.logger.debug(`Emitting consumer.up event for consumer ${this.id}`);
            this.emit('consumer.up', this.id);
            cb(null, true);
        });
    }
    down(cb) {
        this.logger.info('Consumer is down');
        super.down(() => {
            this.logger.debug(`Emitting consumer.down event for consumer ${this.id}`);
            this.emit('consumer.down', this.id);
            if (this.eventBus) {
                this.logger.debug('Delaying event bus shutdown by 1000ms');
                setTimeout(() => {
                    this.logger.debug('Shutting down event bus');
                    this.eventBus?.shutdown((err) => {
                        if (err) {
                            this.logger.warn(`Error during event bus shutdown: ${err.message}`);
                        }
                        this.logger.debug('Event bus shut down');
                        cb(null, true);
                    });
                }, 1000);
            }
            else {
                this.logger.debug('No event bus to shut down');
                cb(null, true);
            }
        });
    }
    getLogger() {
        return this.logger;
    }
    handleError(err) {
        this.logger.error(`Consumer error: ${err.message}`, err);
        this.logger.debug(`Emitting consumer.error event for consumer ${this.id}`);
        this.emit('consumer.error', err, this.id);
        super.handleError(err);
    }
    consume(queue, messageHandler, cb) {
        this.logger.info(`Setting up consumption for queue: ${typeof queue === 'string' ? queue : JSON.stringify(queue)}`);
        const parsedQueueParams = _parseQueueExtendedParams(queue);
        if (parsedQueueParams instanceof Error) {
            this.logger.error(`Failed to parse queue parameters: ${parsedQueueParams.message}`);
            cb(parsedQueueParams);
        }
        else {
            this.logger.debug(`Adding message handler for queue: ${JSON.stringify(parsedQueueParams)}`);
            this.messageHandlerRunner.addMessageHandler(parsedQueueParams, messageHandler, (err) => {
                if (err) {
                    this.logger.error(`Failed to add message handler: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.info(`Successfully set up consumption for queue: ${parsedQueueParams.queueParams.name} (namespace: ${parsedQueueParams.queueParams.ns}${parsedQueueParams.groupId ? `, group: ${parsedQueueParams.groupId}` : ''})`);
                    cb();
                }
            });
        }
    }
    cancel(queue, cb) {
        this.logger.info(`Canceling consumption for queue: ${typeof queue === 'string' ? queue : JSON.stringify(queue)}`);
        const parsedQueueParams = _parseQueueExtendedParams(queue);
        if (parsedQueueParams instanceof Error) {
            this.logger.error(`Failed to parse queue parameters: ${parsedQueueParams.message}`);
            cb(parsedQueueParams);
        }
        else {
            this.logger.debug(`Removing message handler for queue: ${JSON.stringify(parsedQueueParams)}`);
            this.messageHandlerRunner.removeMessageHandler(parsedQueueParams, (err) => {
                if (err) {
                    this.logger.error(`Failed to remove message handler: ${err.message}`);
                    cb(err);
                }
                else {
                    this.logger.info(`Successfully canceled consumption for queue: ${parsedQueueParams.queueParams.name} (namespace: ${parsedQueueParams.queueParams.ns}${parsedQueueParams.groupId ? `, group: ${parsedQueueParams.groupId}` : ''})`);
                    cb();
                }
            });
        }
    }
    getQueues() {
        this.logger.debug('Getting list of queues being consumed');
        const queues = this.messageHandlerRunner.getQueues();
        this.logger.debug(`Consumer is handling ${queues.length} queues: ${JSON.stringify(queues)}`);
        return queues;
    }
}
//# sourceMappingURL=consumer.js.map