import { async, CallbackEmptyReplyError, logger, withRedisClient, } from 'redis-smq-common';
import { ELuaScriptName } from '../../../../../common/redis-client/scripts/scripts.js';
import { redisKeys } from '../../../../../common/redis-keys/redis-keys.js';
import { Configuration } from '../../../../../config/index.js';
import { EMessageProperty, EMessagePropertyStatus, } from '../../../../message/index.js';
import { MessageEnvelope } from '../../../../message/message-envelope.js';
import { _getConsumerQueues } from '../../../consumer/_/_get-consumer-queues.js';
import { ConsumerError } from '../../../errors/index.js';
import { processingQueue } from './processing-queue.js';
import { EMessageUnacknowledgementAction, EMessageUnacknowledgementDeadLetterReason, EMessageUnacknowledgementReason, } from './types/index.js';
export class MessageUnacknowledgement {
    logger;
    redisClient;
    constructor(redisClient) {
        this.redisClient = redisClient;
        this.logger = logger.getLogger(Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
    }
    getMessageUnacknowledgementAction(message, unacknowledgedReason) {
        if (unacknowledgedReason === EMessageUnacknowledgementReason.TTL_EXPIRED ||
            message.getSetExpired()) {
            return {
                action: EMessageUnacknowledgementAction.DEAD_LETTER,
                deadLetterReason: EMessageUnacknowledgementDeadLetterReason.TTL_EXPIRED,
            };
        }
        if (message.isPeriodic()) {
            return {
                action: EMessageUnacknowledgementAction.DEAD_LETTER,
                deadLetterReason: EMessageUnacknowledgementDeadLetterReason.PERIODIC_MESSAGE,
            };
        }
        if (message.hasRetryThresholdExceeded()) {
            return {
                action: EMessageUnacknowledgementAction.DEAD_LETTER,
                deadLetterReason: EMessageUnacknowledgementDeadLetterReason.RETRY_THRESHOLD_EXCEEDED,
            };
        }
        const delay = message.producibleMessage.getRetryDelay();
        return delay
            ? { action: EMessageUnacknowledgementAction.DELAY }
            : { action: EMessageUnacknowledgementAction.REQUEUE };
    }
    getMessageUnacknowledgementParameters(consumerId, messageOrQueue, unacknowledgementReason, done) {
        const [queue, message] = messageOrQueue instanceof MessageEnvelope
            ? [messageOrQueue.getDestinationQueue(), messageOrQueue]
            : [messageOrQueue, null];
        const keys = [];
        const args = [];
        const unacknowledgementStatus = {};
        args.push(JSON.stringify(queue), consumerId);
        const { keyConsumerQueues } = redisKeys.getConsumerKeys(consumerId);
        const { keyQueueProcessing } = redisKeys.getQueueConsumerKeys(queue, consumerId);
        const { keyQueueDL, keyQueueProcessingQueues, keyQueueConsumers, keyQueueProperties, keyQueueDelayed, keyQueueRequeued, } = redisKeys.getQueueKeys(queue, null);
        keys.push(keyQueueProcessing, keyQueueDelayed, keyQueueRequeued, keyQueueDL, keyQueueProcessingQueues, keyQueueConsumers, keyConsumerQueues, keyQueueProperties);
        if (!message) {
            keys.push('');
            args.push('', '', '', unacknowledgementReason, '');
            return done(null, { keys, args, unacknowledgementStatus });
        }
        const messageId = message.getId();
        const { keyMessage } = redisKeys.getMessageKeys(messageId);
        keys.push(keyMessage);
        args.push(messageId);
        const unacknowledgementAction = this.getMessageUnacknowledgementAction(message, unacknowledgementReason);
        const { action } = unacknowledgementAction;
        const messageStatus = action === EMessageUnacknowledgementAction.DEAD_LETTER
            ? EMessagePropertyStatus.DEAD_LETTERED
            : action === EMessageUnacknowledgementAction.REQUEUE
                ? EMessagePropertyStatus.UNACK_REQUEUING
                : EMessagePropertyStatus.UNACK_DELAYING;
        args.push(action, action === EMessageUnacknowledgementAction.DEAD_LETTER
            ? unacknowledgementAction.deadLetterReason
            : '', unacknowledgementReason, messageStatus);
        unacknowledgementStatus[messageId] = unacknowledgementAction;
        done(null, {
            args,
            keys,
            unacknowledgementStatus,
        });
    }
    executeMessageUnacknowledgement(keys, argv, messageHandlingStatus, cb) {
        withRedisClient(this.redisClient, (client, cb) => {
            const { store, expire, queueSize } = Configuration.getSetConfig().messages.store.deadLettered;
            const execKeys = [...keys];
            const execArgs = [
                EMessageUnacknowledgementAction.DELAY,
                EMessageUnacknowledgementAction.REQUEUE,
                EMessageUnacknowledgementAction.DEAD_LETTER,
                Number(store),
                expire,
                queueSize * -1,
                EMessageProperty.STATUS,
                EMessageUnacknowledgementReason.OFFLINE_CONSUMER,
                EMessageUnacknowledgementReason.OFFLINE_MESSAGE_HANDLER,
                ...argv,
            ];
            client.runScript(ELuaScriptName.HANDLE_PROCESSING_QUEUE, execKeys, execArgs, (err, reply) => {
                if (err) {
                    return cb(err);
                }
                if (reply !== 'OK') {
                    return cb(new ConsumerError(reply ? String(reply) : undefined));
                }
                Object.keys(messageHandlingStatus).forEach((messageId) => {
                    const action = messageHandlingStatus[messageId].action ===
                        EMessageUnacknowledgementAction.DEAD_LETTER
                        ? 'dead-lettered'
                        : 'unacknowledged';
                    this.logger.debug(`Message ID ${messageId} has been ${action}.`);
                });
                cb(null, messageHandlingStatus);
            });
        }, cb);
    }
    unacknowledgeMessage(consumerId, msg, unacknowledgementReason, cb) {
        this.getMessageUnacknowledgementParameters(consumerId, msg, unacknowledgementReason, (err, reply) => {
            if (err)
                return cb(err);
            if (!reply)
                return cb(new CallbackEmptyReplyError());
            const { keys, args, unacknowledgementStatus } = reply;
            this.executeMessageUnacknowledgement(keys, args, unacknowledgementStatus ?? {}, cb);
        });
    }
    unacknowledgeMessagesInProcess(consumerId, queues, unacknowledgementReason, cb) {
        const keys = [];
        const args = [];
        const messageHandlingStatus = {};
        withRedisClient(this.redisClient, (client, cb) => {
            async.waterfall([
                (next) => {
                    if (queues === null)
                        _getConsumerQueues(client, consumerId, next);
                    else
                        next(null, queues);
                },
                (queueParams, next) => {
                    if (!queueParams.length) {
                        return next();
                    }
                    async.eachOf(queueParams, (queue, _, done) => {
                        const { keyQueueProcessing } = redisKeys.getQueueConsumerKeys(queue, consumerId);
                        processingQueue.fetchMessageFromProcessingQueue(this.redisClient, keyQueueProcessing, (err, message) => {
                            if (err)
                                return done(err);
                            async.withCallback((cb) => this.getMessageUnacknowledgementParameters(consumerId, message ?? queue, unacknowledgementReason, cb), (reply, cb) => {
                                keys.push(...reply.keys);
                                args.push(...reply.args);
                                Object.assign(messageHandlingStatus, reply.unacknowledgementStatus);
                                cb();
                            }, done);
                        });
                    }, next);
                },
            ], (err) => {
                if (err) {
                    return cb(err);
                }
                if (!keys.length) {
                    return cb(null, {});
                }
                this.executeMessageUnacknowledgement(keys, args, messageHandlingStatus, cb);
            });
        }, cb);
    }
}
//# sourceMappingURL=message-unacknowledgement.js.map