import path from 'path';
import { CallbackEmptyReplyError, CallbackInvalidReplyError, env, logger, Runnable, WorkerResourceGroup, } from 'redis-smq-common';
import { RedisClient } from '../../../../common/redis-client/redis-client.js';
import { ELuaScriptName } from '../../../../common/redis-client/scripts/scripts.js';
import { redisKeys } from '../../../../common/redis-keys/redis-keys.js';
import { Configuration } from '../../../../config/index.js';
import { _fromMessage } from '../../../message/_/_from-message.js';
import { EMessageProperty, EMessagePropertyStatus, } from '../../../message/index.js';
import { ConsumeMessage } from './consume-message/consume-message.js';
import { DequeueMessage } from './dequeue-message/dequeue-message.js';
import { evenBusPublisher } from './even-bus-publisher.js';
export class MessageHandler extends Runnable {
    consumer;
    consumerId;
    queue;
    logger;
    dequeueMessage;
    consumeMessage;
    messageHandler;
    autoDequeue;
    redisClient;
    eventBus;
    workerResourceGroup = null;
    constructor(consumer, redisClient, handlerParams, autoDequeue = true, eventBus) {
        super();
        const { queue, messageHandler } = handlerParams;
        this.consumer = consumer;
        this.consumerId = consumer.getId();
        this.queue = queue;
        this.messageHandler = messageHandler;
        this.logger = logger.getLogger(Configuration.getSetConfig().logger, this.constructor.name.toLowerCase());
        this.logger.info(`Initializing MessageHandler for consumer ${this.consumerId}, queue ${JSON.stringify(this.queue)}`);
        this.logger.debug(`autoDequeue: ${autoDequeue}, messageHandler type: ${typeof messageHandler}`);
        this.autoDequeue = autoDequeue;
        this.redisClient = redisClient;
        this.eventBus = eventBus;
        if (this.eventBus) {
            this.logger.debug('Event bus provided, setting up event bus publisher');
            evenBusPublisher(this, this.eventBus, this.logger);
        }
        else {
            this.logger.debug('No event bus provided, skipping event bus publisher setup');
        }
        this.logger.debug('Initializing DequeueMessage instance');
        this.dequeueMessage = this.initDequeueMessageInstance();
        this.logger.debug('Initializing ConsumeMessage instance');
        this.consumeMessage = this.initConsumeMessageInstance();
        this.logger.debug('Registering system events');
        this.registerSystemEvents();
        this.logger.info(`MessageHandler initialized for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
    }
    initDequeueMessageInstance() {
        this.logger.debug('Creating new DequeueMessage instance');
        const instance = new DequeueMessage(new RedisClient(), this.queue, this.consumer, this.eventBus);
        this.logger.debug('Setting up error handler for DequeueMessage instance');
        instance.on('consumer.dequeueMessage.error', this.onError);
        this.logger.debug('DequeueMessage instance created successfully');
        return instance;
    }
    initConsumeMessageInstance() {
        this.logger.debug('Creating new ConsumeMessage instance');
        const instance = new ConsumeMessage(this.redisClient, this.consumer, this.queue, this.getId(), this.messageHandler, this.eventBus);
        this.logger.debug('Setting up error handler for ConsumeMessage instance');
        instance.on('consumer.consumeMessage.error', this.onError);
        this.logger.debug('ConsumeMessage instance created successfully');
        return instance;
    }
    onMessageReceived = (messageId) => {
        this.logger.debug(`Message received event for message ${messageId}, processing`);
        this.processMessage(messageId);
    };
    onMessageUnacknowledged = (messageId) => {
        this.logger.debug(`Message unacknowledged event for message ${messageId}, requesting next message`);
        this.next();
    };
    onMessageAcknowledged = (messageId) => {
        this.logger.debug(`Message acknowledged event for message ${messageId}, requesting next message`);
        this.next();
    };
    onMessageNext = () => {
        this.logger.debug('Next message event received, requesting next message');
        this.next();
    };
    onError = (err) => {
        if (this.isRunning()) {
            this.logger.error(`Error event received while running: ${err.message}`);
            this.handleError(err);
        }
        else {
            this.logger.debug(`Ignoring error event received during shutdown: ${err.message}`);
        }
    };
    registerSystemEvents = () => {
        this.logger.debug('Registering system event handlers');
        this.dequeueMessage.on('consumer.dequeueMessage.messageReceived', this.onMessageReceived);
        this.dequeueMessage.on('consumer.dequeueMessage.nextMessage', this.onMessageNext);
        this.consumeMessage.on('consumer.consumeMessage.messageUnacknowledged', this.onMessageUnacknowledged);
        this.consumeMessage.on('consumer.consumeMessage.messageAcknowledged', this.onMessageAcknowledged);
        this.logger.debug('All system event handlers registered successfully');
    };
    getLogger() {
        return this.logger;
    }
    handleError(err) {
        if (this.isRunning()) {
            this.logger.error(`MessageHandler error: ${err.message}`, err);
            this.logger.debug(`Emitting consumer.messageHandler.error event for consumer ${this.consumerId}`);
            this.emit('consumer.messageHandler.error', err, this.consumerId, this.queue);
        }
        else {
            this.logger.debug(`Error occurred while not running: ${err.message} (ignoring)`);
        }
        super.handleError(err);
    }
    setUpConsumerWorkers = (cb) => {
        this.logger.debug('Setting up consumer workers');
        const config = Configuration.getSetConfig();
        const { keyQueueWorkersLock } = redisKeys.getQueueKeys(this.queue.queueParams, this.queue.groupId);
        this.logger.debug(`Queue workers lock key: ${keyQueueWorkersLock}`);
        const redisClient = this.redisClient.getInstance();
        if (redisClient instanceof Error) {
            this.logger.error(`Failed to get Redis client: ${redisClient.message}`);
            cb(redisClient);
            return void 0;
        }
        this.logger.debug('Creating WorkerResourceGroup');
        this.workerResourceGroup = new WorkerResourceGroup(redisClient, this.logger, keyQueueWorkersLock);
        this.logger.debug('Setting up error handler for WorkerResourceGroup');
        this.workerResourceGroup.on('workerResourceGroup.error', (err) => {
            this.logger.error(`WorkerResourceGroup error: ${err.message}`);
            this.handleError(err);
        });
        const workersDir = path.resolve(env.getCurrentDir(), '../../workers');
        this.logger.debug(`Loading workers from directory: ${workersDir}`);
        this.workerResourceGroup.loadFromDir(workersDir, { config, queueParsedParams: this.queue }, (err) => {
            if (err) {
                this.logger.error(`Failed to load workers from directory: ${err.message}`);
                cb(err);
            }
            else {
                this.logger.debug('Workers loaded successfully, starting WorkerResourceGroup');
                this.workerResourceGroup?.run((err) => {
                    if (err) {
                        this.logger.error(`Failed to start WorkerResourceGroup: ${err.message}`);
                        this.handleError(err);
                    }
                    else {
                        this.logger.debug('WorkerResourceGroup started successfully');
                    }
                });
                this.logger.debug('Consumer workers setup completed');
                cb();
            }
        });
    };
    shutDownConsumerWorkers = (cb) => {
        if (this.workerResourceGroup) {
            this.logger.debug('Shutting down consumer workers');
            this.workerResourceGroup.shutdown((err) => {
                if (err) {
                    this.logger.warn(`Error during WorkerResourceGroup shutdown: ${err.message} (ignoring)`);
                }
                this.logger.debug('Consumer workers shut down successfully');
                this.workerResourceGroup = null;
                cb();
            });
        }
        else {
            this.logger.debug('No consumer workers to shut down');
            cb();
        }
    };
    goingUp() {
        this.logger.info(`MessageHandler going up for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return super.goingUp().concat([
            (cb) => {
                this.logger.debug('Starting DequeueMessage instance');
                this.dequeueMessage.run((err) => {
                    if (err) {
                        this.logger.error(`Failed to start DequeueMessage: ${err.message}`);
                    }
                    else {
                        this.logger.debug('DequeueMessage started successfully');
                    }
                    cb(err);
                });
            },
            (cb) => {
                this.logger.debug('Starting ConsumeMessage instance');
                this.consumeMessage.run((err) => {
                    if (err) {
                        this.logger.error(`Failed to start ConsumeMessage: ${err.message}`);
                    }
                    else {
                        this.logger.debug('ConsumeMessage started successfully');
                    }
                    cb(err);
                });
            },
            (cb) => {
                if (this.autoDequeue) {
                    this.logger.debug('Auto-dequeue enabled, initiating first dequeue');
                    this.dequeue();
                }
                else {
                    this.logger.debug('Auto-dequeue disabled, skipping initial dequeue');
                }
                cb();
            },
            this.setUpConsumerWorkers,
        ]);
    }
    goingDown() {
        this.logger.info(`MessageHandler going down for consumer ${this.consumerId}, queue ${this.queue.queueParams.name}`);
        return [
            this.shutDownConsumerWorkers,
            (cb) => {
                this.logger.debug('Shutting down DequeueMessage instance');
                this.dequeueMessage.shutdown((err) => {
                    if (err) {
                        this.logger.warn(`Error during DequeueMessage shutdown: ${err.message} (ignoring)`);
                    }
                    else {
                        this.logger.debug('DequeueMessage shut down successfully');
                    }
                    cb();
                });
            },
            (cb) => {
                this.logger.debug('Shutting down ConsumeMessage instance');
                this.consumeMessage.shutdown((err) => {
                    if (err) {
                        this.logger.warn(`Error during ConsumeMessage shutdown: ${err.message} (ignoring)`);
                    }
                    else {
                        this.logger.debug('ConsumeMessage shut down successfully');
                    }
                    cb();
                });
            },
        ].concat(super.goingDown());
    }
    processMessage(messageId) {
        if (!this.isRunning()) {
            this.logger.debug(`Ignoring processMessage call for message ${messageId} as handler is not running`);
            return void 0;
        }
        this.logger.debug(`Processing message ${messageId}`);
        const { keyMessage } = redisKeys.getMessageKeys(messageId);
        this.logger.debug(`Message key: ${keyMessage}`);
        const keys = [keyMessage];
        const argv = [
            EMessageProperty.STATUS,
            EMessageProperty.STATE,
            EMessageProperty.MESSAGE,
            EMessagePropertyStatus.PROCESSING,
        ];
        const redisClient = this.redisClient.getInstance();
        if (redisClient instanceof Error) {
            this.logger.error(`Failed to get Redis client: ${redisClient.message}`);
            this.handleError(redisClient);
            return void 0;
        }
        this.logger.debug(`Running FETCH_MESSAGE_FOR_PROCESSING script for message ${messageId}`);
        redisClient.runScript(ELuaScriptName.FETCH_MESSAGE_FOR_PROCESSING, keys, argv, (err, reply) => {
            if (err) {
                this.logger.error(`Failed to fetch message ${messageId} for processing: ${err.message}`);
                return this.handleError(err);
            }
            if (!reply) {
                this.logger.error(`Empty reply when fetching message ${messageId}`);
                return this.handleError(new CallbackEmptyReplyError());
            }
            if (!Array.isArray(reply)) {
                this.logger.error(`Invalid reply type when fetching message ${messageId}`);
                return this.handleError(new CallbackInvalidReplyError());
            }
            const [state, msg] = reply;
            this.logger.debug(`Message ${messageId} fetched successfully, state: ${state}`);
            const message = _fromMessage(msg, EMessagePropertyStatus.PROCESSING, state);
            this.logger.debug(`Passing message ${messageId} to ConsumeMessage for handling`);
            this.consumeMessage.handleReceivedMessage(message);
        });
    }
    next() {
        this.logger.debug('Requesting next message');
        this.dequeue();
    }
    dequeue() {
        if (this.isRunning()) {
            this.logger.debug('Dequeuing message from queue');
            this.dequeueMessage.dequeue();
        }
        else {
            this.logger.debug('Ignoring dequeue call as handler is not running');
        }
    }
    getQueue() {
        this.logger.debug(`Getting queue parameters: ${JSON.stringify(this.queue)}`);
        return this.queue;
    }
}
//# sourceMappingURL=message-handler.js.map