import { async, CallbackEmptyReplyError, withRedisClient, } from 'redis-smq-common';
import { RedisClient } from '../../../common/redis-client/redis-client.js';
import { _getConsumerGroups } from '../../consumer-groups/_/_get-consumer-groups.js';
import { Message } from '../../message/index.js';
import { _getQueueProperties } from '../../queue/_/_get-queue-properties.js';
import { _parseQueueParams } from '../../queue/_/_parse-queue-params.js';
import { EQueueDeliveryModel, EQueueType, } from '../../queue/index.js';
import { QueueMessagesManagerAbstract } from '../queue-messages-manager/queue-messages-manager-abstract.js';
import { QueueMessagesStorageSet } from '../queue-messages-storage/queue-messages-storage-set.js';
import { QueueAcknowledgedMessages } from './queue-acknowledged-messages.js';
import { QueueDeadLetteredMessages } from './queue-dead-lettered-messages.js';
import { SequentialQueuePendingMessages } from './queue-pending-messages/sequential-queue-pending-messages.js';
import { PriorityQueuePendingMessages } from './queue-pending-messages/priority-queue-pending-messages.js';
import { QueueScheduledMessages } from './queue-scheduled-messages.js';
export class QueueMessages extends QueueMessagesManagerAbstract {
    priorityQueueMessages;
    queuePendingMessages;
    queueDeadLetteredMessages;
    queueScheduledMessages;
    queueAcknowledgedMessages;
    constructor() {
        const redisClient = new RedisClient();
        super(redisClient, new QueueMessagesStorageSet(redisClient), new Message(), 'keyQueueMessages');
        this.priorityQueueMessages = new PriorityQueuePendingMessages();
        this.queuePendingMessages = new SequentialQueuePendingMessages();
        this.queueDeadLetteredMessages = new QueueDeadLetteredMessages();
        this.queueScheduledMessages = new QueueScheduledMessages();
        this.queueAcknowledgedMessages = new QueueAcknowledgedMessages();
        this.logger.debug('QueueMessages initialized');
    }
    countMessagesByStatus(queue, cb) {
        const queueParams = _parseQueueParams(queue);
        if (queueParams instanceof Error) {
            return cb(queueParams);
        }
        withRedisClient(this.redisClient, (client, cb) => this.executeCountingPipeline(client, queueParams, queue, cb), cb);
    }
    executeCountingPipeline(client, queueParams, queue, cb) {
        const count = {
            pending: 0,
            acknowledged: 0,
            scheduled: 0,
            deadLettered: 0,
        };
        async.waterfall([
            (next) => this.getQueueProperties(client, queueParams, next),
            (properties, next) => this.countPendingMessages(client, properties, queue, queueParams, next),
            (pendingCount, next) => {
                count.pending = pendingCount;
                this.queueDeadLetteredMessages.countMessages(queue, next);
            },
            (deadLettered, next) => {
                count.deadLettered = deadLettered;
                this.queueAcknowledgedMessages.countMessages(queue, next);
            },
            (acknowledged, next) => {
                count.acknowledged = acknowledged;
                this.queueScheduledMessages.countMessages(queue, next);
            },
        ], (err, scheduled) => {
            if (err) {
                return cb(err);
            }
            count.scheduled = Number(scheduled);
            return cb(null, count);
        });
    }
    getQueueProperties(client, queueParams, cb) {
        _getQueueProperties(client, queueParams, (err, properties) => {
            if (err) {
                return cb(err);
            }
            if (!properties) {
                return cb(new CallbackEmptyReplyError());
            }
            return cb(null, properties);
        });
    }
    countPendingMessages(client, properties, queue, queueParams, cb) {
        const { queueType, deliveryModel } = properties;
        const countPendingForGroup = (groupId, callback) => {
            if (queueType === EQueueType.PRIORITY_QUEUE) {
                this.priorityQueueMessages.countMessages({ queue, groupId }, callback);
            }
            else {
                this.queuePendingMessages.countMessages({ queue, groupId }, callback);
            }
        };
        if (deliveryModel === EQueueDeliveryModel.PUB_SUB) {
            this.countPendingByGroupId(client, queueParams, countPendingForGroup, cb);
        }
        else {
            countPendingForGroup(null, cb);
        }
    }
    countPendingByGroupId(client, queueParams, countPendingForGroup, cb) {
        _getConsumerGroups(client, queueParams, (err, groups) => {
            if (err) {
                return cb(err);
            }
            const pendingCounts = {};
            async.each(groups ?? [], (groupId, _, next) => {
                countPendingForGroup(groupId, (err, cnt) => {
                    if (err) {
                        return next(err);
                    }
                    pendingCounts[groupId] = Number(cnt);
                    return next();
                });
            }, (err) => cb(err, pendingCounts));
        });
    }
    shutdown(cb) {
        async.series([
            (next) => this.queuePendingMessages.shutdown(next),
            (next) => this.priorityQueueMessages.shutdown(next),
            (next) => this.queueScheduledMessages.shutdown(next),
            (next) => this.queueAcknowledgedMessages.shutdown(next),
            (next) => this.queueDeadLetteredMessages.shutdown(next),
            (next) => super.shutdown(next),
        ], (err) => cb(err));
    }
}
//# sourceMappingURL=queue-messages.js.map